@@ -595,6 +595,7 @@ bool ScriptInterp::scrSelect(void)
 		SEL_LENGTH,
 		SEL_PREFIX,
 		SEL_SUFFIX,
+		SEL_DECIMAL,
 		SEL_VALUE
 	} sm = SEL_NORMAL;
 
@@ -616,6 +617,8 @@ bool ScriptInterp::scrSelect(void)
 			sm = SEL_VALUE;
 		else if(!strnicmp(match, "end", 3))
 			sm = SEL_SUFFIX;
+		else if(!strnicmp(match, "dec", 3))
+			sm = SEL_DECIMAL;
 		else
 		{
 			offset = atoi(match);
@@ -653,6 +656,20 @@ bool ScriptInterp::scrSelect(void)
 			if(atoi(match) == len)
 				return scrGoto();
 			break;
+		case SEL_DECIMAL:
+			match = strchr(match, '.');
+			value = strchr(value, '.');
+
+			if(!match)
+				match = "0";
+			else
+				++match;
+
+			if(!value)
+				value = "0";
+			else
+				++value;
+
 		case SEL_VALUE:
 			if(atoi(match) == atoi(value))
 				return scrGoto();
@@ -706,6 +723,7 @@ bool ScriptInterp::scrGoto(void)
 	int len;
 	Line *skip;
 	Name *scr;
+	bool pvt = true;
 
 	if(!label)
 	{
@@ -733,6 +751,7 @@ bool ScriptInterp::scrGoto(void)
 	len = strlen(label);
 	if(!strncmp(label, "::", 2))
 	{
+		pvt = false;
 		strcpy(namebuf, script[stack].script->name);
 		ext = strstr(namebuf, "::");
 		if(ext)
@@ -743,6 +762,7 @@ bool ScriptInterp::scrGoto(void)
 	}
 	else if(label[len - 1] == ':')
 	{
+		pvt = false;
 		strcpy(namebuf, script[stack].script->name);
 		ext = strstr(namebuf, "::");
 		if(ext)
@@ -763,6 +783,11 @@ bool ScriptInterp::scrGoto(void)
 		error("script-not-found");
 		return true;
 	}
+	if(pvt && !scr->access)
+	{
+		error("script-private");
+		return true;
+	}
 	initKeywords(0);
 	once = true;
 	script[stack].caseflag = false;
@@ -1473,28 +1498,89 @@ bool ScriptInterp::scrPop(void)
 
 bool ScriptInterp::scrReturn(void)
 {
+	Line *line = getScript();
 	bool erflag;
-	char *label = getValue(NULL), *ext;
+	char *label = getValue(NULL), *ext, *var;
 	char namebuf[256];
 	int len;
+	int argc = 0;
 	Name *scr;
+	const char *member = getMember();
+	bool exitflag = false;
+	bool localflag = false;
+	bool topflag = false;
+	bool errflag = false;
+
+	if(!member)
+		member = "0";
+
+	if(!stricmp(member, "exit"))
+		exitflag = true;
+	else if(!stricmp(member, "local"))
+		localflag = true;
+	else if(!stricmp(member, "top"))
+	{
+		localflag = true;
+		topflag = true;
+	}
+
+	len = atoi(member);
+	if(!len)
+		len = getSymbolSize();
+
+	while(argc < line->argc)
+	{
+		if(*line->args[argc++] != '=')
+			continue;
+
+		snprintf(temps[tempidx], getSymbolSize() + 1, 
+			"%s", getContent(line->args[argc]));
+		line->args[argc] = temps[tempidx];
+		if(tempidx++ >= SCRIPT_TEMP_SPACE)
+			tempidx = 0;
+		++argc;
+	}
 
 	do {
 		if(!pull())
+		{
+			errflag = true;
+			if(localflag)
+				break;
+			if(exitflag)
+				scrExit();
 			return true;
+		}
 
-	} while(script[stack].line->loop != 0);
+	} while(script[stack].line->loop != 0 || topflag);
+
+	argc = 0;
+	while(argc < line->argc)
+	{
+		var = line->args[argc++];
+		if(*var != '=')
+			continue;
+
+		ext = line->args[argc++];
+		if(*(++var) == '%')
+			++var;
+
+		setSymbol(var, len);
+		setSymbol(var, ext);
+	}
 
 retry:
 	if(!label)
 	{
-		advance();
+		if(!errflag)
+			advance();
 		return true;
 	}
 
 	if(!*label)
 	{
-		advance();
+		if(!errflag)
+			advance();
 		return true;
 	}
 	if(*label == '^')
@@ -1506,7 +1592,7 @@ retry:
 	{
 		Line *skip =
 			script[stack].script->skip[atoi(label)];
-               	if(!skip)
+               	if(!skip && !errflag)
                        	advance();
                 return true;
 	}
@@ -2435,7 +2521,7 @@ bool ScriptInterp::scrInit(void)
 {
 	Property *prop = NULL;
 	Symbol *sym;
-	int len;
+	int len, rlen = 0;
 	const char *value = getMember();
 	int size = 0;
 	bool number = false;
@@ -2447,7 +2533,11 @@ bool ScriptInterp::scrInit(void)
 	{
 		VAL_NONE,
 		VAL_MIN,
-		VAL_MAX
+		VAL_MAX,
+		VAL_FILL,
+		VAL_RIGHT,
+		VAL_SPLIT,
+		VAL_CENTER
 	}	minmax = VAL_NONE;
 
 	if(value)
@@ -2458,6 +2548,14 @@ bool ScriptInterp::scrInit(void)
 			minmax = VAL_MAX;
 		else if(!stricmp(value, "min"))
 			minmax = VAL_MIN;
+                else if(!stricmp(value, "right"))
+                        minmax = VAL_RIGHT;
+                else if(!stricmp(value, "fill") || !stricmp(value, "left"))
+                        minmax = VAL_FILL;
+                else if(!stricmp(value, "split"))
+                        minmax = VAL_SPLIT;
+                else if(!stricmp(value, "center"))
+                        minmax = VAL_CENTER;
 		else if(!stricmp(value, "val") || !stricmp(value, "num"))
 		{
 			size = 11;
@@ -2511,20 +2609,53 @@ bool ScriptInterp::scrInit(void)
 	sym->data[sym->flags.size] = 0;
 	len = 0;
 
+        while((minmax == VAL_RIGHT || minmax == VAL_CENTER) && rlen < sym->flags.size)
+                sym->data[rlen++] = ' ';
+
 	while(len < sym->flags.size && NULL != (value = getValue(NULL)))
 	{
 		if(minmax == VAL_MIN && atoi(value) >= atoi(sym->data) && len)
 			continue;
 		if(minmax == VAL_MAX && atoi(value) <= atoi(sym->data) && len)
 			continue;
-		if(minmax == VAL_NONE)
+                if(minmax == VAL_RIGHT || minmax == VAL_CENTER)
+                {
+                        rlen -= strlen(value);
+                        if(minmax == VAL_CENTER && rlen > 0)
+                                rlen /= 2;
+
+                        if(rlen < 0)
+                        {
+                                value -= rlen;
+                                rlen = 0;
+                        }
+                        strncpy(sym->data + rlen, value, strlen(value));
+                        if(minmax == VAL_CENTER)
+                                rlen = sym->flags.size;
+                }
+		else if(minmax == VAL_NONE || minmax == VAL_FILL || minmax == VAL_SPLIT)
 			strncpy(sym->data + len, value, sym->flags.size - len);
 		else
 			strncpy(sym->data, value, sym->flags.size);
 		sym->data[sym->flags.size] = 0;
-		len = strlen(sym->data);
+		len = strlen(sym->data) - rlen;
+
+                if(minmax == VAL_SPLIT)
+                {
+                        rlen = len;
+                        while(rlen < sym->flags.size)
+                                sym->data[rlen++] = ' ';
+                        minmax = VAL_RIGHT;
+			len = 0;
+                }
+
 	}
 
+        while(len < sym->flags.size && minmax == VAL_FILL)
+                sym->data[len++] = ' ';
+
+        sym->data[len + rlen] = 0;
+
 	if(number)
 		sprintf(sym->data, "%ld", atol(sym->data));
 	else if(prop)
@@ -2545,16 +2676,20 @@ bool ScriptInterp::scrSet(void)
 	Property *prop = NULL;
 	Symbol *sym;
 	int size = 0;
-	int len;
+	int len, rlen = 0;
 	char *cp, *ep;
 	const char *value;
 	bool number = false;
 
 	enum
 	{
+		VAL_FILL,
+		VAL_RIGHT,
 		VAL_NONE,
 		VAL_MIN,
-		VAL_MAX
+		VAL_MAX,
+		VAL_SPLIT,
+		VAL_CENTER
 	}	minmax = VAL_NONE;
 
 	value = getMember();
@@ -2566,6 +2701,14 @@ bool ScriptInterp::scrSet(void)
 			minmax = VAL_MIN;
 		else if(!stricmp(value, "max"))
 			minmax = VAL_MAX;
+		else if(!stricmp(value, "right"))
+			minmax = VAL_RIGHT;
+		else if(!stricmp(value, "fill") || !stricmp(value, "left"))
+			minmax = VAL_FILL;
+		else if(!stricmp(value, "split"))
+			minmax = VAL_SPLIT;
+		else if(!stricmp(value, "center"))
+			minmax = VAL_CENTER;
 		else if(!strnicmp(value, "val", 3) || !strnicmp(value, "num", 3))
 		{
 			number = true;
@@ -2597,6 +2740,9 @@ bool ScriptInterp::scrSet(void)
 	sym->data[sym->flags.size] = 0;
 	len = 0;
 
+	while((minmax == VAL_RIGHT || minmax == VAL_CENTER) && rlen < sym->flags.size)
+		sym->data[rlen++] = ' ';
+
 	while(len < sym->flags.size && NULL != (value = getValue(NULL)))
 	{
 		if(minmax == VAL_MIN && atoi(value) >= atoi(sym->data) && len)
@@ -2604,14 +2750,43 @@ bool ScriptInterp::scrSet(void)
 
 		if(minmax == VAL_MAX && atoi(value) <= atoi(sym->data) && len)
 			continue;
-		if(minmax == VAL_NONE)
+	
+		if(minmax == VAL_RIGHT || minmax == VAL_CENTER)
+		{
+			rlen -= strlen(value);
+			if(minmax == VAL_CENTER && rlen > 0)
+				rlen /= 2;
+
+			if(rlen < 0)
+			{
+				value -= rlen;
+				rlen = 0;
+			}
+			strncpy(sym->data + rlen, value, strlen(value));
+			if(minmax == VAL_CENTER)
+				rlen = sym->flags.size;
+		}
+		else if(minmax == VAL_NONE || minmax == VAL_FILL || minmax == VAL_SPLIT)
 			strncpy(sym->data + len, value, sym->flags.size - len);
 		else
 			strncpy(sym->data, value, sym->flags.size);
 		sym->data[sym->flags.size] = 0;
-		len = strlen(sym->data);
+		len = strlen(sym->data) - rlen;
+		if(minmax == VAL_SPLIT)
+		{
+			rlen = len;
+			while(rlen < sym->flags.size)
+				sym->data[rlen++] = ' ';
+			minmax = VAL_RIGHT;
+			len = 0;
+		}
 	}
 
+	while(len < sym->flags.size && minmax == VAL_FILL)
+		sym->data[len++] = ' ';
+	
+	sym->data[len + rlen] = 0;	
+
 	if(number)
 		sprintf(sym->data, "%ld", atol(sym->data));
 	else if(prop)
@@ -3244,7 +3419,8 @@ char *ScriptInterp::getContent(char *opt)
 	Property *prop;
 	Symbol *sym;
 	char *ext = strrchr(opt, '.');
-	char *buf;
+	char *buf, *dec;
+	unsigned pos;
 
 	if(!opt)
 		return NULL;
@@ -3288,12 +3464,38 @@ char *ScriptInterp::getContent(char *opt)
 		tempidx = 0;
 	buf = temps[tempidx];
 
-	if(!strnicmp(ext, ".len", 4))
+	if((pos = atoi(ext + 1)) > 0)
+	{
+		dec = sym->data;
+		while(--pos && dec)
+		{
+			dec = strchr(dec, packtoken);
+			if(dec)
+				++dec;
+		}
+		if(dec)
+			strcpy(buf, dec);
+		else
+			strcpy(buf, "");
+		dec = strchr(buf, packtoken);
+		if(dec)
+			*dec = 0;
+	}
+	else if(!strnicmp(ext, ".len", 4))
 		sprintf(buf, "%d", strlen(sym->data));
 	else if(!stricmp(ext, ".size"))
 		sprintf(buf, "%d", sym->flags.size);
 	else if(!strnicmp(ext, ".val", 4) || !strnicmp(ext, ".int", 4))
 		sprintf(buf, "%d", atoi(sym->data));
+	else if(!strnicmp(ext, ".dec", 4))
+	{
+		dec = strchr(sym->data, '.');
+		if(dec)
+			++dec;
+		else
+			dec = "0";
+		sprintf(buf, "%d", atoi(dec));
+	}
 	else if(!stricmp(ext, ".bool"))
 	{
 		strcpy(buf, "false");
