@@ -1452,9 +1452,13 @@ int ScriptImage::compile(istream *str, char *name, const char *scrname)
 	unsigned branch;
 	Method handler;
 	unsigned char loopid, looplevel;
+	unsigned offset = 0;
 	Name *tmpl = NULL, *base = NULL;
 	bool sub = false;
 	streampos pos;
+	bool ignore;
+	bool unsafe = false;
+	bool access = true;
 
 	scrStream = str;
 
@@ -1476,6 +1480,7 @@ compile:
 	script->mask = 0;
 	script->next = index[key];
 	script->mode = Name::ORIGINAL;
+	script->access = access;
 	index[key] = script;
 
 	if(!base)
@@ -1494,15 +1499,27 @@ compile:
 		{
 			cmask = 0;
 			errflag = false;
-			scrStream->getline(buffer, sizeof(buffer) - 1);
+			scrStream->getline(buffer + offset, sizeof(buffer) - 1 - offset);
 			if(scrStream->eof())
 				break;
 
+			bp = strrchr(buffer, '\\');
+			if(bp)
+			{
+				++bp;
+				if(isspace(*bp) || !*bp)
+				{
+					offset = bp - buffer - 1;
+					continue;
+				}
+			}
+
 			bp = buffer;
 		}
 		else
 			then = false;
 
+		offset = 0;
 		++number;
 		while(NULL != (token = getToken()))
 		{
@@ -1661,6 +1678,30 @@ compile:
 			continue;
 		}
 
+		if(!stricmp(token, ".private"))
+		{
+			access = false;
+			continue;
+		}
+
+		if(!stricmp(token, ".public"))
+		{
+			access = true;
+			continue;
+		}
+
+		if(!stricmp(token, ".unsafe"))
+		{
+			unsafe = true;
+			continue;
+		}
+
+		if(!stricmp(token, ".safe"))
+		{
+			unsafe = false;
+			continue;
+		}
+
 		if(!stricmp(token, ".define"))
 		{
 			while(NULL != (token = getToken()))
@@ -1673,17 +1714,25 @@ compile:
 			continue;
 		}
 
-		if(*token == '!')
+		ignore = false;
+		if(*token == '@')
+		{
+			ignore = true;
+			++token;
+		}
+		else if(*token == '!')
 		{
 			errflag = true;
 			++token;
 		}
+			
 		trapflag = false;
 		handler = cmds->getHandler(token);
 		if(!handler)
 		{
 			addmask = submask = 0;
-			slog(Slog::levelError) << token << ": unknown keyword; " << script->name << "(" << number << ")" << endl;
+			if(!ignore && !unsafe)
+				slog(Slog::levelError) << token << ": unknown keyword; " << script->name << "(" << number << ")" << endl;
 			continue;
 		}
 
@@ -1753,7 +1802,8 @@ compile:
 		err = cmds->check(command, line, this);
 		if(err)
 		{
-			slog(Slog::levelError) << command << ": " << err << "; " << script->name << "(" << number << ")" << endl;
+			if(!unsafe)
+				slog(Slog::levelError) << command << ": " << err << "; " << script->name << "(" << number << ")" << endl;
 			continue;
 		}
 
